EU_COUNTRY_CODES = set([
    'AT',  # Austria.
    'BE',  # Belgium.
    'BG',  # Bulgaria.
    'CY',  # Cyprus.
    'CZ',  # Czech Republic.
    'DE',  # Germany.
    'DK',  # Denmark.
    'EE',  # Estonia.
    'ES',  # Spain.
    'FI',  # Finland.
    'FR',  # France.
    'EL', 'GR',  # Greece.
    'HR',  # Croatia.
    'HU',  # Hungary.
    'IE',  # Ireland.
    'IT',  # Italy.
    'LT',  # Lithuania.
    'LU',  # Luxembourg.
    'LV',  # Latvia.
    'MT',  # Malta.
    'NL',  # Netherlands.
    'PL',  # Poland.
    'PT',  # Portugal.
    'RO',  # Romania.
    'SE',  # Sweden.
    'SI',  # Slovenia.
    'SK',  # Slovakia.
])
"""EU country codes.

Represented by ISO 3166-1 alpha-2 country codes.
"""
